@@ -32,8 +32,10 @@ use std::sync::{Arc, Mutex};
 use std::time::Duration;
 
 use openssl::hash::{hash, MessageDigest};
+use protobuf::Message;
 use splinter::{
     consensus::{Proposal, ProposalUpdate},
+    orchestrator::OrchestratableService,
     service::{
         Service, ServiceDestroyError, ServiceError, ServiceMessageContext, ServiceNetworkRegistry,
         ServiceStartError, ServiceStopError,
@@ -315,7 +317,7 @@ impl Service for Scabbard {
         message_bytes: &[u8],
         _message_context: &ServiceMessageContext,
     ) -> Result<(), ServiceError> {
-        let message: ScabbardMessage = protobuf::parse_from_bytes(message_bytes)?;
+        let message: ScabbardMessage = Message::parse_from_bytes(message_bytes)?;
 
         match message.get_message_type() {
             ScabbardMessage_Type::CONSENSUS_MESSAGE => self
@@ -360,6 +362,16 @@ impl Service for Scabbard {
     }
 }
 
+impl OrchestratableService for Scabbard {
+    fn as_service(&self) -> &dyn Service {
+        self
+    }
+
+    fn clone_box(&self) -> Box<dyn OrchestratableService> {
+        Box::new(self.clone())
+    }
+}
+
 fn compute_db_paths(
     service_id: &str,
     circuit_id: &str,
